@@ -1,5 +1,5 @@
 /* Input routines.
-   Copyright (C) 1989-1998, 2002-2004 Free Software Foundation, Inc.
+   Copyright (C) 1989-1998, 2002-2004, 2011 Free Software Foundation, Inc.
    Written by Douglas C. Schmidt <schmidt@ics.uci.edu>
    and Bruno Haible <bruno@clisp.org>.
 
@@ -545,6 +545,11 @@ Input::read_input ()
                   option.set (NULLSTRINGS);
                 else
 
+                if (is_define_declaration (line, line_end, lineno,
+                                           "constants-prefix", &arg))
+                  option.set_constants_prefix (arg);
+                else
+
                 if (is_define_declaration (line, line_end, lineno,
                                            "word-array-name", &arg))
                   option.set_wordlist_name (arg);
